@@ -271,8 +271,7 @@ class cExportItem(gmBusinessDBObject.cBusinessDBObject):
 		if not success:
 			return None
 
-		if filename.endswith('.dat'):
-			filename = gmMimeLib.adjust_extension_by_mimetype(filename)
+		filename = gmMimeLib.adjust_extension_by_mimetype(filename)
 		if passphrase is None:
 			return filename
 
@@ -315,14 +314,17 @@ class cExportItem(gmBusinessDBObject.cBusinessDBObject):
 				date_before_type = True,
 				name_first = False
 			)
-		target_mime = 'application/pdf' if convert2pdf else None
-		target_ext = '.pdf' if convert2pdf else None
+		if convert2pdf:
+			target_mime = 'application/pdf'
+			target_ext = '.pdf'
+		else:
+			target_mime = None
+			target_ext = None
 		part_fname = part.save_to_file (
 			filename = filename,
 			target_mime = target_mime,
 			target_extension = target_ext,
-			ignore_conversion_problems = False,
-			adjust_extension = True
+			ignore_conversion_problems = False
 		)
 		if part_fname is None:
 			_log.error('cannot save document part to file')
@@ -337,7 +339,7 @@ class cExportItem(gmBusinessDBObject.cBusinessDBObject):
 			verbose = _cfg.get(option = 'debug'),
 			remove_unencrypted = True
 		)
-		removed = gmTools.remove_file(filename)
+		removed = gmTools.remove_file(part_fname)
 		if enc_filename is None:
 			_log.error('cannot encrypt')
 			return False
@@ -963,7 +965,7 @@ class cExportArea(object):
 		# - export mugshot
 		mugshot = pat.document_folder.latest_mugshot
 		if mugshot is not None:
-			mugshot_fname = mugshot.save_to_file(directory = doc_dir, adjust_extension = True)
+			mugshot_fname = mugshot.save_to_file(directory = doc_dir)
 			fname = os.path.split(mugshot_fname)[1]
 			html_data['mugshot_url'] = os.path.join(DOCUMENTS_SUBDIR, fname)
 			html_data['mugshot_alt'] =_('patient photograph from %s') % gmDateTime.pydt_strftime(mugshot['date_generated'], '%B %Y')
