@@ -1,5 +1,66 @@
 # -*- coding: utf-8 -*-
-"""GNUmed person searching code."""
+"""GNUmed person searching code.
+
+	How-To search:
+		name:		Leonard Spock "Pointy-Ears" NIMOY-ZALDANA
+		DOB:		Oct 15 1975
+		ID:			12345678
+		ext ID:		abcd-13d-4d
+		ext ID:		#abcd-13d-4d
+
+	find patient by name (first, last, nick):
+		'leon'
+		'spo'
+		'nimo'
+		'zald'
+		'pointy'
+
+	find patient by lastname:
+		'NIMO'
+		'ZALD'
+		'nIm,'
+		'Zal,'
+
+	find patient by firstname:
+		', spo'
+		',Leon'
+
+	find patient by nickname:
+		'!point'
+		'!ear'
+
+	find patient by firstname and lastname:
+		'Leonard NIMO'
+		'spock ZALDA'
+
+	find patient by fragment anywhere inside name:
+		'...ock'
+		'...ldan'
+		'...moY'
+		'...Nar'
+		'...Ear'
+
+	find patient by several name parts:
+		'leon nim'
+		'spo zal'
+		'leon nim zald'
+
+	find patient by GNUmed ID:
+		'12345678' (also searches by DOB)
+		'#12345678' (also searches by external ID)
+
+	find patient by DOB:
+		'15101975' (also searches for GNUmed ID)
+		'*15101975'
+		'*15/10/1975'
+		'*15-10-1975'
+		'*15.10.1975'
+		'*15 10 1975'
+
+	find patient by external ID:
+		'##abcd-13d-4d'		(finds ID "abcd-13d-4d")
+		'## #abcd-13d-4d'	(finds ID "#abcd-13d-4d")
+"""
 #============================================================
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
@@ -115,19 +176,34 @@ class cPatientSearcher_SQL:
 			return aString
 
 		# umlauts
-		normalized =    aString.replace('Ä', '(Ä|AE|Ae|A|E)')
-		normalized = normalized.replace('Ö', '(Ö|OE|Oe|O)')
+		normalized = aString.replace('Ö', '(Ö|OE|Oe|O)')
 		normalized = normalized.replace('Ü', '(Ü|UE|Ue|U)')
-		normalized = normalized.replace('ä', '(ä|ae|e|a)')
 		normalized = normalized.replace('ö', '(ö|oe|o)')
 		normalized = normalized.replace('ü', '(ü|ue|u|y)')
 		normalized = normalized.replace('ß', '(ß|sz|ss|s)')
 
 		# common soundalikes
 		# - René, Desiré, Inés ...
+		normalized = normalized.replace('e', '***DUMMY***')
 		normalized = normalized.replace('é', '***DUMMY***')
 		normalized = normalized.replace('è', '***DUMMY***')
-		normalized = normalized.replace('***DUMMY***', '(é|e|è|ä|ae)')
+		normalized = normalized.replace('ë', '***DUMMY***')
+		normalized = normalized.replace('ê', '***DUMMY***')
+		normalized = normalized.replace('ä', '***DUMMY***')
+		normalized = normalized.replace('æ', '***DUMMY***')
+		normalized = normalized.replace('ae', '***DUMMY***')
+		normalized = normalized.replace('***DUMMY***', '(e|é|è|ë|ê|ä|æ|ae)')
+		# upper case
+		normalized = normalized.replace('E', '***DUMMY***')
+		normalized = normalized.replace('É', '***DUMMY***')
+		normalized = normalized.replace('È', '***DUMMY***')
+		normalized = normalized.replace('Ë', '***DUMMY***')
+		normalized = normalized.replace('Ê', '***DUMMY***')
+		normalized = normalized.replace('Ä', '***DUMMY***')
+		normalized = normalized.replace('Æ', '***DUMMY***')
+		normalized = normalized.replace('Ae', '***DUMMY***')
+		normalized = normalized.replace('AE', '***DUMMY***')
+		normalized = normalized.replace('***DUMMY***', '(E|É|È|Ë|Ê|Ä|Æ|Ae|AE)')
 
 		# FIXME: missing i/a/o - but uncommon in German
 		normalized = normalized.replace('v', '***DUMMY***')
@@ -168,297 +244,419 @@ class cPatientSearcher_SQL:
 	#--------------------------------------------------------
 	def __queries_for_firstname_with_comma(self, raw):
 		"""Generate search queries for [ , <alpha> ] search terms."""
-		if regex.match(",\s*\w+$", raw.strip()) is None:
+		if not raw.startswith(','):
 			return []
+
+		raw = raw.lstrip(' ,')
+		if not raw.isalpha():
+			return []
+
 		_log.debug("[%s]: a firstname" % raw)
-		tmp = self._normalize_soundalikes(raw.strip(' ,'))
-		cmd = """
+		normalized = self._normalize_soundalikes(raw)
+		cmd = """-- find patients by ",firstname"
 			SELECT DISTINCT ON (pk_identity) * FROM (
-				SELECT *, %(match)s AS match_type FROM ((
+				SELECT *, %(match)s AS match_type FROM (
 					SELECT d_vap.*
 					FROM dem.names, dem.v_active_persons d_vap
-					WHERE dem.names.firstnames ~ %(first)s and d_vap.pk_identity = dem.names.id_identity
-				) union all (
-					SELECT d_vap.*
-					FROM dem.names, dem.v_active_persons d_vap
-					WHERE dem.names.firstnames ~ %(first_w_caps)s and d_vap.pk_identity = dem.names.id_identity
-				) union all (
-					SELECT d_vap.*
-					FROM dem.names, dem.v_active_persons d_vap
-					WHERE lower(dem.names.firstnames) ~ lower(%(first)s) and d_vap.pk_identity = dem.names.id_identity
-				)) AS super_list ORDER BY lastnames, firstnames, dob
+					WHERE dem.names.firstnames ~* %(first)s and d_vap.pk_identity = dem.names.id_identity
+				) AS super_list ORDER BY lastnames, firstnames, dob
 			) AS sorted_list"""
 		args = {
 			'match': _('first name'),
-			'first': '^' + tmp,
-			'first_w_caps': '^' + gmTools.capitalize(tmp, mode = gmTools.CAPS_NAMES)
+			'first': '\m' + normalized
 		}
 		return [{'cmd': cmd, 'args': args}]
 
 	#--------------------------------------------------------
 	def __queries_for_lastname_with_comma(self, raw):
 		"""Generate search queries for [ <alpha> , ] search terms."""
-		if regex.match("\w+\s*,$", raw) is None:
+		if not raw.endswith(','):
 			return []
+
+		raw = raw.rstrip(' ,')
+		if not raw.isalpha():
+			return []
+
 		_log.debug("[%s]: a lastname" % raw)
-		tmp = self._normalize_soundalikes(raw.strip(' ,'))
-		cmd = """
+		normalized = self._normalize_soundalikes(raw)
+		cmd = """-- find patients by "lastname,":
 			SELECT DISTINCT ON (pk_identity) * FROM (
 				SELECT *, %(match)s AS match_type FROM ((
 					SELECT d_vap.*
 					FROM dem.names, dem.v_active_persons d_vap
-					WHERE dem.names.lastnames ~ %(last)s and d_vap.pk_identity = dem.names.id_identity
-				) union all (
-					SELECT d_vap.*
-					FROM dem.names, dem.v_active_persons d_vap
-					WHERE dem.names.lastnames ~ %(last_w_caps)s and d_vap.pk_identity = dem.names.id_identity
-				) union all (
-					SELECT d_vap.*
-					FROM dem.names, dem.v_active_persons d_vap
-					WHERE lower(dem.names.lastnames) ~ lower(%(last)s) and d_vap.pk_identity = dem.names.id_identity
+					WHERE dem.names.lastnames ~* %(last)s and d_vap.pk_identity = dem.names.id_identity
 				)) AS super_list ORDER BY lastnames, firstnames, dob
 			) AS sorted_list"""
 		args = {
 			'match': _('last name'),
-			'last': '^' + tmp,
-			'last_w_caps': '^' + gmTools.capitalize(tmp, mode=gmTools.CAPS_NAMES)
+			'last': '\m%s' % normalized
 		}
 		return [{'cmd': cmd, 'args': args}]
 
 	#--------------------------------------------------------
 	def __queries_for_LASTNAME(self, raw):
 		"""Generate search queries for [ <ALPHA> ] search terms."""
-		if regex.match("\w+$", raw) is None:
+		if not raw.isalpha():
 			return []
+
 		if raw != raw.upper():
-			# not all UPPERCASE
 			return []
+
 		_log.debug("[%s]: a lastname" % raw)
-		tmp = self._normalize_soundalikes(raw)
-		cmd = """
+		normalized = self._normalize_soundalikes(raw)
+		cmd = """-- find patients by "LASTNAME":
 			SELECT DISTINCT ON (pk_identity) * FROM (
-				SELECT *, %(match)s AS match_type FROM ((
+				SELECT *, %(match)s AS match_type FROM (
 					SELECT d_vap.*
 					FROM dem.names, dem.v_active_persons d_vap
-					WHERE dem.names.lastnames ~ %(last_w_caps)s and d_vap.pk_identity = dem.names.id_identity
-				) union all (
-					SELECT d_vap.*
-					FROM dem.names, dem.v_active_persons d_vap
-					WHERE lower(dem.names.lastnames) ~ lower(%(last)s) and d_vap.pk_identity = dem.names.id_identity
-				)) AS super_list ORDER BY lastnames, firstnames, dob
+					WHERE dem.names.lastnames ~* %(last)s and d_vap.pk_identity = dem.names.id_identity
+				) AS super_list ORDER BY lastnames, firstnames, dob
 			) AS sorted_list"""
 		args = {
 			'match': _('last name'),
-			'last': '^' + tmp,
-			'last_w_caps': '^' + gmTools.capitalize(tmp.lower(), mode=gmTools.CAPS_NAMES)
+			'last': '\m' + normalized
 		}
 		return [{'cmd': cmd, 'args': args}]
 
+	#--------------------------------------------------------
+	def __queries_for_name_fragment(self, raw):
+		if not raw.startswith('...'):
+			return []
+
+		raw = raw.lstrip('.')
+		if not raw.isalpha():
+			return []
+
+		_log.debug("[%s]: a singular name fragment" % raw)
+		fragment = self._normalize_soundalikes(raw)
+		SQL = """-- find patients by name fragments anywhere in name:
+			SELECT DISTINCT ON (pk_identity) * FROM (
+				SELECT *, %(match)s AS match_type FROM (
+					SELECT d_vap.*
+					FROM dem.names JOIN dem.v_active_persons d_vap ON (d_vap.pk_identity = dem.names.id_identity)
+					WHERE
+						dem.names.lastnames ~* %(fragment)s
+						OR dem.names.firstnames ~* %(fragment)s
+						OR dem.names.preferred ~* %(fragment)s
+				) AS super_list
+				ORDER BY lastnames, firstnames, dob
+			) AS sorted_list"""
+		args = {
+			'match': _('name part'),
+			'fragment': fragment
+		}
+		return [{'cmd': SQL, 'args': args}]
+
+	#--------------------------------------------------------
+	def __queries_for_nickname(self, raw):
+		if not raw.startswith('!'):
+			return []
+
+		raw = raw[1:]
+		_log.debug("[%s]: a nickname" % raw)
+		nick = self._normalize_soundalikes(raw)
+		SQL = """-- find patients by name fragments anywhere in name:
+			SELECT DISTINCT ON (pk_identity) * FROM (
+				SELECT *, %(match)s AS match_type FROM (
+					SELECT d_vap.*
+					FROM dem.names JOIN dem.v_active_persons d_vap ON (d_vap.pk_identity = dem.names.id_identity)
+					WHERE dem.names.preferred ~* %(nick)s
+				) AS super_list
+				ORDER BY lastnames, firstnames, dob
+			) AS sorted_list"""
+		args = {
+			'match': _('nickname'),
+			'nick': nick
+		}
+		return [{'cmd': SQL, 'args': args}]
+
+	#--------------------------------------------------------
+	def __queries_for_singular_name_part(self, raw):
+		if not raw.isalpha():
+			return []
+
+		if len(raw) < 3:
+			return []
+
+		_log.debug("[%s]: a singular name part" % raw)
+		name = self._normalize_soundalikes(raw)
+		SQL = """-- find patients by name part even inside multi-part names:
+			SELECT DISTINCT ON (pk_identity) * FROM (
+				SELECT *, %(match)s AS match_type FROM (
+					SELECT d_vap.*
+					FROM dem.names JOIN dem.v_active_persons d_vap ON (d_vap.pk_identity = dem.names.id_identity)
+					WHERE dem.names.lastnames ~* %(name)s OR dem.names.firstnames ~* %(name)s OR dem.names.preferred ~* %(name)s
+				) AS super_list
+				ORDER BY lastnames, firstnames, dob
+			) AS sorted_list"""
+		args = {
+			'match': _('name part'),
+			'name': '\m%s' % name
+		}
+		return [{'cmd': SQL, 'args': args}]
+
+	#--------------------------------------------------------
+	def __queries_for_several_name_parts(self, raw):
+		parts = regex.split('[,\.\-\s]+', raw)
+		print(parts)
+		for p in parts:
+			if not p.isalpha():
+				return []
+
+		parts = [ p for p in parts if len(p) > 1 ]
+		if len(parts) < 2:
+			return []
+
+		_log.debug("[%s]: several name parts" % parts)
+		parts = [ '\m' + self._normalize_soundalikes(p) for p in parts ]
+		SQL = """-- find patients by name parts even inside multi-part names:
+			SELECT DISTINCT ON (pk_identity) * FROM (
+				SELECT *, %(match)s AS match_type FROM (
+					SELECT d_vap.*
+					FROM dem.names JOIN dem.v_active_persons d_vap ON (d_vap.pk_identity = dem.names.id_identity)
+					WHERE dem.names.lastnames ~* ANY(%(parts)s) AND dem.names.firstnames ~* ANY(%(parts)s)
+					-- AND dem.names.preferred ~* ANY(%(parts)s)
+				) AS super_list
+				ORDER BY lastnames, firstnames, dob
+			) AS sorted_list"""
+		args = {
+			'match': _('name parts'),
+			'parts': parts
+		}
+		return [{'cmd': SQL, 'args': args}]
+
 	#--------------------------------------------------------
 	def __queries_for_LAST_and_first(self, raw):
 		"""Generate search queries for [ <ALPHA> <alpha> ] or [ <alpha> <ALPHA> ] search terms."""
-		if regex.match("\w+\s+\w+$", raw) is None:
+#		if regex.match('^\w+\s+\w+$', raw) is None:
+#			return []
+#		if raw == raw.upper():
+#			# ALL caps
+#			return []
+#		if raw == raw.lower():
+#			# ALL lowercase
+#			return []
+#		parts = [ p for p in regex.split('\s+', raw) ]
+		parts = raw.split()
+		if len(parts) != 2:
 			return []
-		if raw == raw.upper():
-			# ALL caps
+
+		for p in parts:
+			if not p.isalpha():
+				return []
+
+		p1_upcase = parts[0] == parts[0].upper()
+		p2_upcase = parts[1] == parts[1].upper()
+		if p1_upcase and p2_upcase:
 			return []
-		if raw == raw.lower():
-			# ALL lowercase
+
+		if True not in [p1_upcase, p2_upcase]:
 			return []
-		parts = [ p for p in regex.split('\s+', raw) ]
-		last = None
-		if parts[0] == parts[0].upper():
+
+		if p1_upcase:
 			last = parts[0]
 			first = parts[1]
-		if parts[1] == parts[1].upper():
+		else:
 			last = parts[1]
 			first = parts[0]
+
+#		last = None
+#		if parts[0] == parts[0].upper():
+#			last = parts[0]
+#			first = parts[1]
+#		if parts[1] == parts[1].upper():
+#			last = parts[1]
+#			first = parts[0]
 		# found no UPPERCASE
-		if last is None:
-			return []
+#		if last is None:
+#			return []
+
 		_log.debug("[%s]: <LASTNAME firstname> or firstname LASTNAME" % raw)
 		last = self._normalize_soundalikes(last)
 		first = self._normalize_soundalikes(first)
-		cmd = """
-			SELECT DISTINCT ON (pk_identity) * FROM (
-				SELECT *, %(match)s AS match_type FROM ((
+		cmd = """-- get patients by "LASTNAME firstname"
+			SELECT DISTINCT ON (pk_identity) *
+			FROM (
+				SELECT *, %(match)s AS match_type
+				FROM (
 					SELECT d_vap.*
 					FROM dem.names, dem.v_active_persons d_vap
 					WHERE
-						dem.names.lastnames ~ %(last)s
+						dem.names.lastnames ~* %(last)s
 							AND
-						dem.names.firstnames ~ %(first)s
+						dem.names.firstnames ~* %(first)s
 							AND
 						d_vap.pk_identity = dem.names.id_identity
-				) union all (
-					SELECT d_vap.*
-					FROM dem.names, dem.v_active_persons d_vap
-					WHERE
-						lower(dem.names.lastnames) ~ lower(%(last)s)
-							AND
-						lower(dem.names.firstnames) ~ lower(%(first)s)
-							AND
-						d_vap.pk_identity = dem.names.id_identity
-				)) AS super_list ORDER BY lastnames, firstnames, dob
+				) AS super_list
+				ORDER BY lastnames, firstnames, dob
 			) AS sorted_list"""
 		args = {
 			'match': _('LASTNAME and firstname'),
-			'last': '^' + last,
-			'first': '^' + first
+			'last': '\m' + last,
+			'first': '\m' + first
 		}
 		return [{'cmd': cmd, 'args': args}]
 
 	#--------------------------------------------------------
-	def _generate_simple_query(self, raw):
-		"""Compose queries if search term seems unambigous."""
-		queries = []
+	def __queries_for_external_id(self, raw):
+		eid = raw.lstrip('#').lstrip()
+		if not eid:
+			return []
 
-		#raw = raw.strip(' ,;')
+		_log.debug("[%s]: an external ID", raw)
+		SQL = """-- find patients by external ID:
+			SELECT d_vap.*, %(match_type)s::text AS match_type
+			FROM dem.lnk_identity2ext_id d_li2ei, dem.v_active_persons d_vap
+			WHERE d_vap.pk_identity = d_li2ei.id_identity AND d_li2ei.external_id ~* %(ext_id)s
+			ORDER BY lastnames, firstnames, dob;"""
+		args = {
+			'match_type': _('external patient ID'),
+			'ext_id': eid
+		}
+		return [{'cmd': SQL, 'args': args}]
+
+	#--------------------------------------------------------
+	def __queries_for_pk(self, raw):
+		raw = raw.lstrip('#')
+		if not raw.isdigit():
+			return []
+
+		_log.debug("[%s]: a PK" % raw)
+		SQL = """-- find patient by PK:
+			SELECT *, %(match_type)s::text AS match_type
+			FROM dem.v_active_persons
+			WHERE pk_identity = %(pk)s
+			ORDER BY lastnames, firstnames, dob"""
+		args = {
+			'match_type': _('internal patient ID'),
+			'pk': raw
+		}
+		return [{'cmd': SQL, 'args': args}]
+
+	#--------------------------------------------------------
+	def __queries_for_pk_or_external_id(self, raw):
+		_log.debug("[%s]: a GNUmed patient PK or an external ID", raw)
+		if raw.startswith('##'):
+			return self.__queries_for_external_id(raw)
+
+		queries = self.__queries_for_pk(raw)
+		if queries:
+			return queries
+
+		return self.__queries_for_external_id(raw)
+
+	#--------------------------------------------------------
+	def __queries_for_DOB(self, raw):
+		if len(raw) < 8:
+			return []
+
+		dob = raw.lstrip('*')
+		dob = dob.replace('.', '-')
+		dob = dob.replace('/', '-')
+		dob = dob.replace(' ', '-')
+		dob = dob.replace('\t', '-')
+		digits = dob.replace('-', '')
+		if len(digits) < 6:
+			return []
+
+		if not digits.isdigit():
+			return []
+
+		_log.debug("[%s]: a DOB (-> %s)" % (raw, dob))
+		SQL = """-- find patients by DOB:
+			SELECT *, %(match_type)s AS match_type
+			FROM dem.v_active_persons
+			WHERE dem.date_trunc_utc('day'::text, dob) = dem.date_trunc_utc('day'::text, %(dob)s::timestamp with time zone)
+			ORDER BY lastnames, firstnames, dob"""
+		args = {
+			'match_type': _('date of birth'),
+			'dob': dob
+		}
+		return [{'cmd': SQL, 'args': args}]
+
+	#--------------------------------------------------------
+	def generate_simple_query(self, raw):
+		"""Compose queries if search term seems unambiguous."""
 		raw = raw.strip()
+		if not raw:
+			return []
+
+		# "*..." - DOB
+		if raw.startswith('*'):
+			queries = self.__queries_for_DOB(raw)
+			if queries:
+				return queries
+
+		# "##.*" - external ID
+		if raw.startswith('##'):
+			queries = self.__queries_for_external_id(raw)
+			if queries:
+				return queries
+
+		# "#.*" - GNUmed patient PK or external ID
+		if raw.startswith('#'):
+			queries = self.__queries_for_pk_or_external_id(raw)
+			if queries:
+				return queries
+
+		# "...<alpha>" - *anywhere* inside first or last or nick
+		if raw.startswith('...'):
+			queries = self.__queries_for_name_fragment(raw)
+			if queries:
+				return queries
+
+		# "!.*" - nickname
+		if raw.startswith('!'):
+			queries = self.__queries_for_nickname(raw)
+			if queries:
+				return queries
 
 		# "<digits>" - GNUmed patient PK or DOB
-		if regex.match("^(\s|\t)*\d+(\s|\t)*$", raw):
-			_log.debug("[%s]: a PK or DOB" % raw)
-			queries.append ({
-				'cmd': "SELECT *, %s::text AS match_type FROM dem.v_active_persons WHERE pk_identity = %s ORDER BY lastnames, firstnames, dob",
-				'args': [_('internal patient ID'), raw]
-			})
-			if len(raw) > 7:	# DOB needs at least 8 digits
-				queries.append ({
-					'cmd': "SELECT *, %s::text AS match_type FROM dem.v_active_persons WHERE dem.date_trunc_utc('day'::text, dob) = dem.date_trunc_utc('day'::text, %s::timestamp with time zone) ORDER BY lastnames, firstnames, dob",
-					'args': [_('date of birth'), raw.replace(',', '.')]
-				})
-			queries.append ({
-				'cmd': """
-					SELECT vba.*, %s::text AS match_type
-					FROM
-						dem.lnk_identity2ext_id li2ext_id,
-						dem.v_active_persons vba
-					WHERE
-						vba.pk_identity = li2ext_id.id_identity and lower(li2ext_id.external_id) ~* lower(%s)
-					ORDER BY
-						lastnames, firstnames, dob
-				""",
-				'args': [_('external patient ID'), raw]
-			})
+		if raw.isdigit():
+			queries = self.__queries_for_pk(raw)
+			queries.extend(self.__queries_for_DOB(raw))
 			return queries
 
-		# "<d igi ts>" - DOB or patient PK
+		# "<d igi ts>" - DOB
 		if regex.match("^(\d|\s|\t)+$", raw):
-			_log.debug("[%s]: a DOB or PK" % raw)
-			queries.append ({
-				'cmd': "SELECT *, %s::text AS match_type FROM dem.v_active_persons WHERE dem.date_trunc_utc('day'::text, dob) = dem.date_trunc_utc('day'::text, %s::timestamp with time zone) ORDER BY lastnames, firstnames, dob",
-				'args': [_('date of birth'), raw.replace(',', '.')]
-			})
-			tmp = raw.replace(' ', '')
-			tmp = tmp.replace('\t', '')
-			queries.append ({
-				'cmd': "SELECT *, %s::text AS match_type FROM dem.v_active_persons WHERE pk_identity LIKE %s%%",
-				'args': [_('internal patient ID'), tmp]
-			})
-			return queries
-
-		# "#<di git  s>" - GNUmed patient PK
-		if regex.match("^(\s|\t)*#(\d|\s|\t)+$", raw):
-			_log.debug("[%s]: a PK" % raw)
-			tmp = raw.replace('#', '')
-			tmp = tmp.strip()
-			tmp = tmp.replace(' ', '')
-			tmp = tmp.replace('\t', '')
-			# this seemingly stupid query ensures the PK actually exists
-			queries.append ({
-				'cmd': "SELECT *, %s::text AS match_type FROM dem.v_active_persons WHERE pk_identity = %s ORDER BY lastnames, firstnames, dob",
-				'args': [_('internal patient ID'), tmp]
-			})
-			return queries
-
-		# "##<di git s or c-hars>" - external ID"
-		# but might also be an external ID
-#			tmp = raw.replace('#', '')
-#			tmp = tmp.strip()
-#			tmp = tmp.replace(' ',  '***DUMMY***')
-#			tmp = tmp.replace('\t', '***DUMMY***')
-#			tmp = tmp.replace('***DUMMY***', '(\s|\t|-|/)*')
-#			queries.append ({
-#				'cmd': """
-#					SELECT vba.*, %s::text AS match_type FROM dem.lnk_identity2ext_id li2ext_id, dem.v_active_persons vba
-#					WHERE vba.pk_identity = li2ext_id.id_identity and lower(li2ext_id.external_id) ~* lower(%s)
-#					ORDER BY lastnames, firstnames, dob""",
-#				'args': [_('external patient ID'), tmp]
-#			})
-
-		# "#<di git s or c-hars>" - external ID
-		if regex.match("^(\s|\t)*#.+$", raw):
-			_log.debug("[%s]: an external ID" % raw)
-			tmp = raw.replace('#', '')
-			tmp = tmp.strip()
-			tmp = tmp.replace(' ',  '***DUMMY***')
-			tmp = tmp.replace('\t', '***DUMMY***')
-			tmp = tmp.replace('-',  '***DUMMY***')
-			tmp = tmp.replace('/',  '***DUMMY***')
-			tmp = tmp.replace('***DUMMY***', '(\s|\t|-|/)*')
-			queries.append ({
-				'cmd': """
-					SELECT
-						vba.*,
-						%s::text AS match_type
-					FROM
-						dem.lnk_identity2ext_id li2ext_id,
-						dem.v_active_persons vba
-					WHERE
-						vba.pk_identity = li2ext_id.id_identity
-							AND
-						lower(li2ext_id.external_id) ~* lower(%s)
-					ORDER BY
-						lastnames, firstnames, dob""",
-				'args': [_('external patient ID'), tmp]
-			})
-			return queries
+			queries = self.__queries_for_DOB(raw)
+			if queries:
+				return queries
 
 		# digits interspersed with "./-" or blank space - DOB
-		if regex.match("^(\s|\t)*\d+(\s|\t|\.|\-|/)*\d+(\s|\t|\.|\-|/)*\d+(\s|\t|\.)*$", raw):
-			_log.debug("[%s]: a DOB" % raw)
-			tmp = raw.strip()
-			while '\t\t' in tmp: tmp = tmp.replace('\t\t', ' ')
-			while '  ' in tmp: tmp = tmp.replace('  ', ' ')
-			# apparently not needed due to PostgreSQL smarts...
-			#tmp = tmp.replace('-', '.')
-			#tmp = tmp.replace('/', '.')
-			queries.append ({
-				'cmd': "SELECT *, %s AS match_type FROM dem.v_active_persons WHERE dem.date_trunc_utc('day'::text, dob) = dem.date_trunc_utc('day'::text, %s::timestamp with time zone) ORDER BY lastnames, firstnames, dob",
-				'args': [_('date of birth'), tmp.replace(',', '.')]
-			})
-			return queries
+		if regex.match("^\d+(\s|\t|\.|\-|/)*\d+(\s|\t|\.|\-|/)*\d+\.*$", raw):
+			queries = self.__queries_for_DOB(raw)
+			if queries:
+				return queries
 
-		# " , <alpha>" - first name
+		# ", <alpha>" - first name
 		queries = self.__queries_for_firstname_with_comma(raw)
-		if len(queries) > 0:
+		if queries:
 			return queries
 
-		# "<alpha>, " - last name
+		# "<alpha>," - last name
 		queries = self.__queries_for_lastname_with_comma(raw)
-		if len(queries) > 0:
+		if queries:
 			return queries
 
 		# "<ALPHA>" - last name
 		queries = self.__queries_for_LASTNAME(raw)
-		if len(queries) > 0:
+		if queries:
 			return queries
 
 		# "<alpha alpha>" - first last or last first, depending on UPPERCASE
 		queries = self.__queries_for_LAST_and_first(raw)
-		if len(queries) > 0:
+		if queries:
 			return queries
 
-		# "*|$<...>" - DOB
-		if regex.match("\s*(\*|\$).+$", raw):
-			_log.debug("[%s]: a DOB" % raw)
-			tmp = raw.replace('*', '')
-			tmp = tmp.replace('$', '')
-			queries.append ({
-				'cmd': "SELECT *, %s AS match_type FROM dem.v_active_persons WHERE dem.date_trunc_utc('day'::text, dob) = dem.date_trunc_utc('day'::text, %s::timestamp with time zone) ORDER BY lastnames, firstnames, dob",
-				'args': [_('date of birth'), tmp.replace(',', '.')]
-			})
+		# "<alpha>" - first OR last name OR nick
+		queries = self.__queries_for_singular_name_part(raw)
+		if queries:
+			return queries
+
+		# "<alpha> <alpha> <alpha> ..." - first OR last name OR nick
+		queries = self.__queries_for_several_name_parts(raw)
+		if queries:
 			return queries
 
 		return []
@@ -579,16 +777,14 @@ class cPatientSearcher_SQL:
 							d_vap.*,
 							%s::text AS match_type
 						FROM
-							dem.v_active_persons d_vap,
-							dem.names n
+							dem.v_active_persons d_vap
+								left join dem.names n on (n.id_identity = d_vap.pk_identity)
 						WHERE
-							d_vap.pk_identity = n.id_identity
-								AND
 							-- name_parts[0]
-							lower(n.firstnames || ' ' || n.lastnames) ~* lower(%s)
+							(n.firstnames || ' ' || n.lastnames) ~* %s
 								AND
 							-- name_parts[1]
-							lower(n.firstnames || ' ' || n.lastnames) ~* lower(%s)""",
+							(n.firstnames || ' ' || n.lastnames) ~* lower(%s)""",
 				'args': [_('name'), name_parts[0], name_parts[1]]
 			})
 			return queries
@@ -648,7 +844,7 @@ class cPatientSearcher_SQL:
 			return []
 
 		# check to see if we get away with a simple query ...
-		queries = self._generate_simple_query(search_term)
+		queries = self.generate_simple_query(search_term)
 		if len(queries) > 0:
 			_log.debug('[%s]: search term with a simple, unambigous structure' % search_term)
 			return queries
@@ -1054,30 +1250,49 @@ if __name__ == '__main__':
 	#--------------------------------------------------------
 	def test_generate_simple_query():
 		searcher = cPatientSearcher_SQL()
-		print("testing _generate_simple_query()")
+		print("testing generate_simple_query()")
 		print("----------------------------")
 		data = [
-			'51234', '1 134 153', '#13 41 34', '#3-AFY322.4', '22-04-1906', '1235/32/3525',
-			', tiberiu',# firstname
-			'KIRK',		# lastname
-			'kirk,',	# lastname
-			'KIR tib',	# LAST first
-			'Tib KI'	# first LAST
+			'jam tib, kir',
+			'...tiber',
+			'...iber',
+			'kirk',
+			'#123',
+			'#1 23',
+			'##123',
+			'##1 23',
+			'#abc',
+			'##abc',
+			'## #abc',
+			'123',
+			'1 23',
+			'1 23 1974'
+			'!Pointy'
+#			'51234', '1 134 153', '#13 41 34', '#3-AFY322.4', '22-04-1906', '1235/32/3525',
+#			', tiberiu',# firstname
+#			'KIRK',		# lastname
+#			'kirk,',	# lastname + comma
+#			'Kirk,',	# lastname + comma
+#			'KIR tib',	# LAST first
+#			'Tib KI'	# first LAST
 		]
 		for fragment in data:
 			print("fragment:", fragment)
-			qs = searcher._generate_simple_query(fragment)
+			qs = searcher.generate_simple_query(fragment)
 			for q in qs:
 				print('')
-				print(" match on:", q['args'])
 				print(" query   :", q['cmd'])
+				print(" match on:", q['args'])
+				print(gmPG2.run_ro_queries(queries = [q]))
 			input('press [ENTER] to continue')
 			print("============")
 
 	#--------------------------------------------------------
-	test_generate_simple_query()
+	gmPG2.request_login_params(setup_pool = True)
+
+	#test_generate_simple_query()
 	#test_patient_search_queries()
-	#test_ask_for_patient()
+	test_ask_for_patient()
 	#test_search_by_dto()
 
 #============================================================
